package LibertyCheck;
use strict;
use warnings;
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(process_lc);

sub process_lc {
  my ($aref, $sizes, $checkselfcapture) = @_;
  my $self = setup($sizes);
  my $normalsize = $sizes->[0] < 19 && $sizes->[1] < 19;

  for my $move (@{$aref}) {
    my ($colour, $x, $y) = @{$move};
    analysemove($self, $colour, $x, $y, $checkselfcapture) unless $x == 19 && $y == 19 && $normalsize;
  }

  return exportaref($self, $aref);
}

sub setup {
  my $sizes = shift;
  my $self = {};
  $self->{colourmap}  = { B => 1, AB => 1, W => 2, AW => 2 };
  $self->{ocolourmap} = { B => 2, AB => 2, W => 1, AW => 1 };
  $self->{cellfarm}   = makecellfarm($sizes->[0]+1, $sizes->[1]+1);
  return $self;
}

sub exportaref {
  my ($self, $aref) = @_;
  my @a;

  for (@{$aref}) {
    my ($colour, $x, $y) = @{$_};
    if ($self->{colourmap}->{$colour} == $self->{cellfarm}->{$x, $y}) {
      push @a, $_;
      $self->{cellfarm}->{$x, $y} = 0; # so only first move is pushed onto @a 
    }
  }

  return \@a;
  
}

sub analysemove {
  my ($self, $colour, $x, $y, $checkselfcapture) = @_;
  my $xo = $self->{colourmap}->{$colour};
  my $cellfarm = $self->{cellfarm};
  $cellfarm->{$x, $y} = $xo;
  if (checkforcaptures($cellfarm, $self->{ocolourmap}->{$colour}, $x, $y) && $checkselfcapture) {
    checklibs($cellfarm, $xo, $x, $y);
  }
}

sub checkforcaptures {
  my ($cellfarm, $xo, $x, $y) = @_;
  my $nocaptures = 1;

  for ([1,0], [0,1], [-1,0], [0,-1]) {
    my ($xx, $yy) = ($_->[0] + $x, $_->[1] + $y);
    $nocaptures = 0 if $cellfarm->{$xx, $yy} == $xo && checklibs($cellfarm, $xo, $xx, $yy);
  }

  return $nocaptures;
}

sub checklibs {
  my ($cellfarm, $xo, @toVisit) = @_;
  my %group;

  while (@toVisit) {
    my ($x, $y) = splice @toVisit, 0, 2;
    $group{$x, $y} = undef;

    for ([1,0], [0,1], [-1,0], [0,-1]) {
      my ($xx, $yy) = ($_->[0] + $x, $_->[1] + $y);
      my $contents = $cellfarm->{$xx, $yy};
      return 0 if !$contents;
      push @toVisit, $xx, $yy if $contents == $xo && !exists $group{$xx, $yy};
    }

  }

  map { $cellfarm->{$_} = 0 } keys %group;
  return 1;
}

sub printcellfarm { # debugging
  my ($ref, $sizex, $sizey) = @_;
  my %map = (-1 => '-', 0 => '.', 2 => 'o', 1 => 'x');
  for my $y (-1..$sizey) {
    for my $x (-1..$sizex) {
      print $map{$ref->{$x, $y}}, ' ';
    }
    print "\n";
  }
  print "\n";
}

sub makecellfarm {
  my ($sizex, $sizey) = @_;
  my $ref;

  for my $x (-1..$sizex) {
    for my $y (-1..$sizey) {
      if ($x == -1 || $y == -1 || $x == $sizex || $y == $sizey) {
        $ref->{$x, $y} = -1; # offboard
      } else {
        $ref->{$x, $y} = 0;
      }
    }
  }

  return $ref;
}

1;
